﻿import socket
import threading
import tkinter as tk
from tkinter import scrolledtext, messagebox, StringVar
from tkinter import Menu, ttk
import json
import os

class ClientApp:
    def __init__(self, root, host='147.185.221.19', port=42439):
        self.root = root
        self.root.title("DMconnect")
        self.root.geometry("600x400")
        self.host = host
        self.port = port
        self.client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        style = ttk.Style()
        style.configure("TButton", padding=6)
        style.configure("TLabel", padding=5)

        self.chat_display = scrolledtext.ScrolledText(root, state='disabled', wrap='word', width=70, height=15, font=("Arial", 10), bg="black", foreground="#D2D2D2")
        self.chat_display.grid(row=0, column=0, padx=5, pady=3, sticky="nsew")

        self.input_frame = tk.Frame(root, bg="black")
        self.input_frame.grid(row=1, column=0, padx=5, pady=3, sticky="ew")

        self.input_text = tk.Text(self.input_frame, height=1, wrap='word', font=("Arial", 10), bg="black", foreground="#D2D2D2", insertbackground="white")
        self.input_text.pack(side=tk.LEFT, fill=tk.X, expand=True)
        self.input_text.bind("<KeyRelease>")
        self.input_text.bind("<Return>", self.handle_return_key)
        self.input_text.bind("<Shift-Return>", self.handle_shift_return_key)

        self.context_menu = Menu(root, tearoff=0)
        self.context_menu.add_command(label="Копировать", command=self.copy_message)
        self.context_menu.add_command(label="Вставить", command=self.paste_message)
        self.context_menu.add_command(label="Цитировать", command=self.quote_message)

        self.chat_display.bind("<Button-3>", self.show_context_menu)

        root.grid_rowconfigure(0, weight=1)
        root.grid_rowconfigure(1, weight=0)
        root.grid_columnconfigure(0, weight=1)

        self.max_chars = 1000
        self.stop_event = threading.Event()

        self.start_connection()

    def start_connection(self):
        try:
            self.client_socket.connect((self.host, self.port))
            self.start_receiving_thread()
        except Exception as e:
            self.add_message("Ошибка подключения: " + str(e))

    def add_message(self, message):
        self.chat_display.config(state='normal')
        self.chat_display.insert(tk.END, message + '\n')
        self.chat_display.yview(tk.END)
        self.chat_display.config(state='disabled')

    def send_message(self):
        message = self.input_text.get("1.0", tk.END).strip()
        if message.strip():
            self.send_normal_message(message)

        self.input_text.delete("1.0", tk.END)

    def send_normal_message(self, message):
        try:
            self.client_socket.send(message.encode('utf-8'))
            if message.lower() == 'exit':
                self.stop_event.set()
                self.client_socket.close()
                self.root.quit()
        except Exception as e:
            self.add_message("Ошибка отправки сообщения: " + str(e))

    def handle_return_key(self, event):
        if not event.state & 0x0001:
            self.send_message()
        return "break"

    def handle_shift_return_key(self, event):
        self.input_text.insert(tk.END, '\n')
        return "break"

    def receive_messages(self):
        while not self.stop_event.is_set():
            try:
                message = self.client_socket.recv(1024).decode('utf-8').strip()
                if not message:
                    break
                self.add_message(message)
            except Exception as e:
                if not self.stop_event.is_set():
                    self.add_message("Ошибка при получении сообщения: " + str(e))
                break

    def start_receiving_thread(self):
        receive_thread = threading.Thread(target=self.receive_messages, daemon=True)
        receive_thread.start()

    def show_context_menu(self, event):
        try:
            self.context_menu.post(event.x_root, event.y_root)
        except tk.TclError:
            pass

    def copy_message(self):
        try:
            selected_text = self.chat_display.get(tk.SEL_FIRST, tk.SEL_LAST)
            self.root.clipboard_clear()
            self.root.clipboard_append(selected_text)
        except tk.TclError:
            pass

    def paste_message(self):
        try:
            clipboard_text = self.root.clipboard_get()
            self.input_text.insert(tk.END, clipboard_text)
        except tk.TclError:
            pass

    def quote_message(self):
        try:
            selected_text = self.chat_display.get(tk.SEL_FIRST, tk.SEL_LAST)
            self.input_text.insert(tk.END, " > " + str(selected_text) + "\n")
        except tk.TclError:
            pass

if __name__ == "__main__":
    root = tk.Tk()
    app = ClientApp(root)
    root.mainloop()

